@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2005-2006 Sun Microsystems Inc.
+# Copyright 2005-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -22,12 +22,13 @@ level, which is held in the debugLevel field.  All other methods take
 a debug level, which is compared to the current debug level to
 determine if the content should be output."""
 
-__id__        = "$Id: debug.py,v 1.29 2006/06/10 00:54:47 wwalker Exp $"
-__version__   = "$Revision: 1.29 $"
-__date__      = "$Date: 2006/06/10 00:54:47 $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: debug.py 2535 2007-07-09 21:43:14Z wwalker $"
+__version__   = "$Revision: 2535 $"
+__date__      = "$Date: 2007-07-09 17:43:14 -0400 (Mon, 09 Jul 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
+import sys
 import traceback
 
 # Used to turn off all debugging.
@@ -143,6 +144,14 @@ def println(level, text = ""):
     - text: the text to print (default is a blank line)
     """
 
+    # Check to see if debugging is not enabled for the calling module.
+    #
+    frame = sys._getframe(1)
+    name = frame.f_globals["__name__"]
+    module = sys.modules[name]
+    if module.__dict__.has_key("_enableDebug") and not module._enableDebug:
+        return
+
     if level >= debugLevel:
         if debugFile:
             debugFile.writelines([text,"\n"])
@@ -195,5 +204,39 @@ def printDetails(level, indent, accessible, includeApp=True):
     - includeApp: if True, include information about the app
     """
 
-    if accessible:
+    if level >= debugLevel and accessible:
         println(level, accessible.toString(indent, includeApp))
+
+# The following code has been borrowed from the following URL:
+# 
+# http://www.dalkescientific.com/writings/diary/archive/2005/04/20/tracing_python_code.html
+#
+import linecache
+
+def traceit(frame, event, arg):
+    """Line tracing utility to output all lines as they are executed by
+    the interpreter.  This is to be used by sys.settrace and is for 
+    debugging purposes.
+   
+    Arguments:
+    - frame: is the current stack frame
+    - event: 'call', 'line', 'return', 'exception', 'c_call', 'c_return',
+             or 'c_exception'
+    - arg:   depends on the event type (see docs for sys.settrace)
+    """ 
+
+    if event == "line":
+        lineno = frame.f_lineno
+        filename = frame.f_globals["__file__"]
+        if (filename.endswith(".pyc") or
+            filename.endswith(".pyo")):
+            filename = filename[:-1]
+        name = frame.f_globals["__name__"]
+        if name == "gettext" \
+           or name == "locale" \
+           or name == "posixpath" \
+           or name == "UserDict":
+            return traceit
+        line = linecache.getline(filename, lineno)
+        println(LEVEL_ALL, "TRACE %s:%s: %s" % (name, lineno, line.rstrip()))
+    return traceit
